#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if BITMAP_LEVELS == 1
#include "../util/bitmap1L.h"
#else
#include "../util/bitmap.h"
#endif

#include "../util/coordinates_encoding/coordinates_storage.h"
#include "../util/coordinates_encoding/encoder.h"

#ifndef ENTER_DATA_UTIL_H
#include "../util/EnterDataUtil.h"
#endif
	
#ifndef MATH_UTILS_H
#include "../util/math_utils.h"
#endif

#ifndef TIEMPO_H
#include "../util/tiempo.h"
#endif

#define INTERSECTOR_SIZE 5000000 //max number of MBRs

class rdt_wt {
  public:
    rdt_wt(INPUT_MBR *input_data, uint input_size, uint blockSize, char *encoder, int scaleFactor);
    ~rdt_wt();
    int executeQueryWT(INPUT_MBR *mbr, long *result, uint *resultSize);
    uint getNumberOfElements() {return number_of_elements;}
    uint memoryUsage();
    
  private:
    uint number_of_elements;

    CoordinatesStorage *fax;
    CoordinatesStorage *fay;
    CoordinatesStorage *fcx;
    CoordinatesStorage *fcy;
    
    uint *idsCX;
    uint *idsCY;

    uint wtSize;
    uint wtLevels;
    uint wtLevelSize;//next power of two after points_size: 2^(ceil(log_2(points_size)))
    uint *lonwt; //longitudes wavelet tree
    uint *latwt; //latitudes wavelet tree
    bitmap lonwtBitmap; //bitmap of the longitude wavelet tree to implement block and superblock structures (for rank and select)
    bitmap latwtBitmap; //bitmap of the longitude wavelet tree to implement block and superblock structures (for rank and select)

    //auxiliary structures (work space)
    uint *subChunkUpperLimits;
    uint *newSubChunkUpperLimits;
    uint *candidatePositions;
    uint *intersector;
    uint intersectorSize;

    //Aux functions
    int translatePermut(INPUT_MBR *mbrsOrderedByCoords, uint type, uint element, COORD *otherCoords);
    int getChunk(int element, uint chunkSize);
    int translateQuery(MBR *mbr, uint *maxax, uint *mincx, uint *maxay, uint *mincy);
    inline void clearIntersector() {
        register uint i;
	for(i = 0; i < intersectorSize; i++) {
	  intersector[i] = 0;
	}
    }
};
/*
    //statistics
    int numReportByLevel[2][30];
    float candidates1[2];
    float candidates2[2];
    float results[2];
    float totalResults;
*/
